//=============================================================================
// includes
//=============================================================================

#include "shader.h"

#include <memory.h>

#include <iostream>
#include <fstream>
using namespace std;

#include <glh/glh_extensions.h>
#include <glh/glh_genext.h>


//=============================================================================
// implementation
//=============================================================================

CShader::CShader()
{
    m_vertexShader   = 0;
    m_fragmentShader = 0;
    m_shaderProg     = 0;
}


CShader::~CShader()
{
}


bool CShader::loadShaders(const char* vertexName, const char* fragmentName)
{
    glh_init_extensions("GL_ARB_shader_objects");
    glh_init_extensions("GL_ARB_vertex_shader");
    glh_init_extensions("GL_ARB_fragment_shader");
    glh_init_extensions("GL_ARB_multitexture");

    const unsigned int maxSize = 16384;
    char* vertex = NULL;
    char* fragment = NULL;

    // load vertex shader
    ifstream vertexFile(vertexName, ios::in|ios::binary|ios::ate);
    if (!vertexFile.is_open()) return false;

    ifstream::pos_type size = vertexFile.tellg();
    vertex = new char[size+ifstream::pos_type(1)];
    vertexFile.seekg(0, ios::beg);
    vertexFile.read(vertex, size);
    vertexFile.close();
    vertex[size] = '\0';

    // load fragment shader
    ifstream fragmentFile(fragmentName, ios::in|ios::binary|ios::ate);
    if (!fragmentFile.is_open())
    {
        cout << "unable to load fragment file " << fragmentName;
        delete[] vertex;
        return false;
    }

    size = fragmentFile.tellg();
    fragment = new char[size+ifstream::pos_type(1)];
    fragmentFile.seekg(0, ios::beg);
    fragmentFile.read(fragment, size);
    fragmentFile.close();
    fragment[size] = '\0';

    m_vertexShader   = glCreateShaderObjectARB(GL_VERTEX_SHADER);
    m_fragmentShader = glCreateShaderObjectARB(GL_FRAGMENT_SHADER);

    const char* pVertex   = vertex;
    const char* pFragment = fragment;
    glShaderSourceARB(m_vertexShader, 1, &pVertex, NULL);
    glShaderSourceARB(m_fragmentShader, 1, &pFragment, NULL);

    glCompileShaderARB(m_vertexShader);
    glCompileShaderARB(m_fragmentShader);

    printShaderInfoLog(m_vertexShader);
    printShaderInfoLog(m_fragmentShader);

    m_shaderProg = glCreateProgramObjectARB();
    glAttachObjectARB(m_shaderProg, m_vertexShader);
    glAttachObjectARB(m_shaderProg, m_fragmentShader);

    glLinkProgramARB(m_shaderProg);
    printProgramInfoLog(m_shaderProg);

    delete[] fragment;
    delete[] vertex;

    return true;
}


bool CShader::use()
{
    if (m_shaderProg == 0) return false;
    
    glUseProgramObjectARB(m_shaderProg);

    return true;
}


bool CShader::setUniform(const char* name, const int value)
{
    GLint location = glGetUniformLocationARB(m_shaderProg, name);
    if (location < 0) return false;

    glUniform1iARB(location, value);

    return true;
}


bool CShader::setUniform(const char* name, const float value)
{
    GLint location = glGetUniformLocationARB(m_shaderProg, name);
    if (location < 0) return false;

    glUniform1fARB(location, value);

    return true;
}


bool CShader::setUniform(const char* name, const float value1, const float value2)
{
    GLint location = glGetUniformLocationARB(m_shaderProg, name);
    if (location < 0) return false;

    glUniform2fARB(location, value1, value2);

    return true;
}


bool CShader::setUniform(const char* name, const float value1, const float value2, const float value3)
{
    GLint location = glGetUniformLocationARB(m_shaderProg, name);
    if (location < 0) return false;

    glUniform3fARB(location, value1, value2, value3);

    return true;
}


bool CShader::disable()
{
    if (m_shaderProg == 0) return false;

    glUseProgramObjectARB(0);

    return true;
}


void CShader::printShaderInfoLog(GLuint obj)
{
    int infologLength = 0;
    int charsWritten  = 0;
    char *infoLog;

    glGetObjectParameterivARB(obj, GL_OBJECT_INFO_LOG_LENGTH_ARB, &infologLength);

    if (infologLength > 0)
    {
        infoLog = new char [infologLength];
        glGetInfoLogARB(obj, infologLength, &charsWritten, infoLog);
        cout << infoLog << endl;
        delete[] infoLog;
    }
}


void CShader::printProgramInfoLog(GLuint obj)
{
    int infologLength = 0;
    int charsWritten  = 0;
    char *infoLog;

    glGetObjectParameterivARB(obj, GL_OBJECT_INFO_LOG_LENGTH_ARB, &infologLength);

    if (infologLength > 0)
    {
        infoLog = new char [infologLength];
        glGetInfoLogARB(obj, infologLength, &charsWritten, infoLog);
        cout << infoLog << endl;
        delete[] infoLog;
    }
}

